#define GAME_EVENT_SPAWN 1 
#define GAME_EVENT_END   2 

typedef struct {
	s16 x1;
	s16 x2;
	s16 y1;
	s16 y2;
} game_bb;

typedef struct {
	u8 object;
	u8 move;
	u8 swap;
	s16 x;
	s16 p;
	s16 y;
	s16 target_x;
	s16 target_p;
	s16 target_y;
	u8 speed_x;
	u8 speed_p;
	u8 speed_y;
	u8 moveType;
} game_objectSpawn;


typedef struct {
	u8 type;
	u16 frame;
	game_objectSpawn spawn;
} game_event;

char game_textScore[4] = "000";

u8 game_SRAM_offset = 4;

u8 game_speed_forward = 1;
u8 game_speed_back = 1;
u8 game_speed_up = 1;
u8 game_speed_down = 1;

u16 game_frameOfDeath = 0;

u16 game_lastScore = 0;

const game_event * game_events;
u16 game_eventCount = 0;
u16 game_frameCount = 0;
u16 game_lastEvent = 0;


u8 * game_victims;
u8 * game_victims_move_death;
u8 * game_victims_swap_death;
game_bb * game_victims_bb;

u8 * game_enemies;
u8 game_enemyCount = 0;
u8 * game_enemy_swap_kill;
game_bb * game_enemy_bb;

u8 game_victimCount = 0;
s16 game_lastVictimCollision = -1;
s16 game_lastEnemyCollision = -1;

u8 game_player = 0;
u8 game_swap_player_eat = 0;
u8 game_swap_player_die = 0;
u8 game_move_player_die = 0;




void game_movePlayer(HWORD pad, u8 player)
{
	if((pad&K_RU) || (pad&K_LU))
	{
		if(ani_objects[player].y > 35)
		{
			ani_objects[player].y = ani_objects[player].y - game_speed_up;
		}
	}
	
	if((pad&K_RD) || (pad&K_LD))
	{
		//if(ani_objects[player].y < 160)
		if(ani_objects[player].y < 140)
		{
			ani_objects[player].y = ani_objects[player].y + game_speed_down;
		}
	}
	
	if((pad&K_RL) || (pad&K_LL))
	{
		if(ani_objects[player].x > 0)
		{
			ani_objects[player].x = ani_objects[player].x - game_speed_back;
		}
	}
	
	if((pad&K_RR) || (pad&K_LR))
	{
		if(ani_objects[player].x < 340)
		{
			ani_objects[player].x = ani_objects[player].x + game_speed_forward;
		}
	}
	
}

void game_triggerEatAnimation(u16 swap)
{
	ani_activateSwap(swap);
}

void game_triggerDeathAnimation(u16 swap, u16 move)
{
	ani_objectmove[move].target_x = ani_objects[ani_objectmove[move].object].x - 250;
	ani_activateSwap(swap);
	ani_activateMove(move);
}

void game_triggerVictimDeathAnimation(u16 swap, u16 move)
{
	ani_objectmove[move].target_x = ani_objects[ani_objectmove[move].object].x - 250;
	ani_activateSwap(swap);
	ani_activateMove(move);
	
}

void game_triggerEnemyKillAnimation(u16 swap)
{
	ani_activateSwap(swap);
}

u8 game_checkCollision(u8 player, u8 obstacle, game_bb bb)
{

	if(ani_objects[player].x + ani_objects[player].w >= (ani_objects[obstacle].x  + bb.x1) 
	   && ani_objects[player].x + ani_objects[player].w <= (ani_objects[obstacle].x  + bb.x2)
	   && ani_objects[player].y > (ani_objects[obstacle].y + bb.y1) 
	   && ani_objects[player].y <= (ani_objects[obstacle].y + bb.y2) 
	)
	{
		return 1;
	}
		
	return 0;
}

u16 game_checkVicitimCollision(void)
{
	u8 i = 0;
	u8 victimCollision = 0;
	u8 score = 0;
	
	if(game_frameOfDeath==0)
	{
		for(i=0; i < game_victimCount; i++)
		{
			if(ani_objects[game_victims[i]].active==1)
			{
				if(game_checkCollision(game_player, game_victims[i], game_victims_bb[i])==1)
				{
					victimCollision = 1;
					
					if(game_lastVictimCollision==-1)
					{
						game_triggerEatAnimation(game_swap_player_eat);
						game_triggerVictimDeathAnimation(game_victims_swap_death[i], game_victims_move_death[i]);
						score = 1;
						suse_playSound(sndKill);
						//score = 100;
					}
					game_lastVictimCollision = game_victims[i];
					
					return score;
				}
			}
		}
		
		if(victimCollision==0)
		{
			game_lastVictimCollision = -1;
		}
	}
	
	return 0;
}

void game_checkEnemyCollision(void)
{
	u8 i = 0;
	u8 enemyCollision = 0;
	
	if(game_frameOfDeath==0)
	{
		for(i=0; i < game_enemyCount; i++)
		{
			if(ani_objects[game_enemies[i]].active==1)
			{
				if(game_checkCollision(game_player, game_enemies[i], game_enemy_bb[i])==1)
				{
					enemyCollision = 1;
					
					if(game_lastEnemyCollision==-1)
					{
						game_triggerDeathAnimation(game_swap_player_die, game_move_player_die);
						game_triggerEnemyKillAnimation(game_enemy_swap_kill[i]);
						game_frameOfDeath = game_frameCount + 150;
						//suse_playSound(sndPickupPickup);
						suse_playSound(sndEnemyDeath);
						
					}
					game_lastEnemyCollision = game_enemies[i];
					
					break;
				}
			}
		}
		
		if(enemyCollision==0)
		{
			game_lastEnemyCollision = -1;
		}
	}
	
}

void game_setGameScore(u16 score)
{
	char * buffer;
		

	if(score > 999)
	{
		score = 999;
		game_lastScore = score;
	}
	
	if(score < 10)
	{
		buffer = itoa(score,10,1);
		game_textScore[2] = buffer[0];
		game_textScore[1] = '0';
		game_textScore[0] = '0';
	}
	else if(score < 100)
	{
		buffer = itoa(score,10,2);
		game_textScore[2] = buffer[1];
		game_textScore[1] = buffer[0];
		game_textScore[0] = '0';
	}
	else if(score < 1000)
	{
		buffer = itoa(score,10,3);
		game_textScore[2] = buffer[2];
		game_textScore[1] = buffer[1];
		game_textScore[0] = buffer[0];
	}
}

void saveLastScoreToSRAM()
{
	u16 lastSavedScore=0;
	
	lastSavedScore = mem_getFromSRAM(game_SRAM_offset) << 8;
	lastSavedScore = ( lastSavedScore | mem_getFromSRAM(game_SRAM_offset+1));
	
	if(game_lastScore > lastSavedScore)
	{
		mem_setToSRAM(game_SRAM_offset,     (u8)((game_lastScore & 0xFF00) >> 8) );
		mem_setToSRAM(game_SRAM_offset + 1, (u8)(game_lastScore & 0x00FF) );
	}
}

u8 game_handleEvents(u8 status)
{
	u16 i = 0;
	
	if(game_frameOfDeath==game_frameCount)
	{
		saveLastScoreToSRAM();
		return STATUS_GAMEOVERSCREEN;
	}
	
	for(i=game_lastEvent; i < game_eventCount; i++)
	{
		if(game_events[i].type==GAME_EVENT_END)
		{
			if(game_events[i].frame == game_frameCount)
			{
				saveLastScoreToSRAM();
				return STATUS_CREDITS;
			}
		}
		else if(game_events[i].type==GAME_EVENT_SPAWN)
		{
			if(game_events[i].frame == game_frameCount)
			{
				game_lastEvent = i+1;
				
				ani_objectmove[game_events[i].spawn.move].target_x = game_events[i].spawn.target_x;
				ani_objectmove[game_events[i].spawn.move].target_p = game_events[i].spawn.target_p;
				ani_objectmove[game_events[i].spawn.move].target_y = game_events[i].spawn.target_y;
				ani_objectmove[game_events[i].spawn.move].delay = 0;
				//ani_objectmove[game_events[i].spawn.move].type = game_events[i].spawn.moveType;
				ani_objectmove[game_events[i].spawn.move].start_x = game_events[i].spawn.x;
				ani_objectmove[game_events[i].spawn.move].start_p = game_events[i].spawn.p;
				ani_objectmove[game_events[i].spawn.move].start_y = game_events[i].spawn.y;
				ani_objects[game_events[i].spawn.object].x =  game_events[i].spawn.x;
				ani_objects[game_events[i].spawn.object].p =  game_events[i].spawn.p;
				ani_objects[game_events[i].spawn.object].y =  game_events[i].spawn.y;
				ani_objectmove[game_events[i].spawn.move].speed_x =  game_events[i].spawn.speed_x;
				ani_objectmove[game_events[i].spawn.move].speed_p =  game_events[i].spawn.speed_p;
				ani_objectmove[game_events[i].spawn.move].speed_y =  game_events[i].spawn.speed_y;
				ani_activateMove(game_events[i].spawn.move);
				ani_activateSwap(game_events[i].spawn.swap);
				ani_activateObject(game_events[i].spawn.object);
			}
			else
			{
				return status;
			}
		}
	}
	return status;
}

u8 game_display(HWORD pad, u8 player, u8 status)
{
	u16 score = 0;
	u8 ret = status;
	
	game_frameCount++;
	ret = game_handleEvents(status);
	
	game_movePlayer(pad, player);
	
	if((pad&K_A))
	{
		//game_triggerDeathAnimation(L1_SWAP_PLAYER_DEATH, L1_MOVE_PLAYER_DEATH);
		//ani_objectmove[7].active = 0;
		//ani_objectmove[11].active = 1;
	}
	
	score = game_lastScore + game_checkVicitimCollision();
	if(score != game_lastScore)
	{
		game_lastScore = score;
		game_setGameScore(score);
		vbTextOutS(5, 0, 0, game_textScore);
	}
	
	game_checkEnemyCollision();
	
	return ret;
}